/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.log.handler;

import java.io.File;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.Source;

import com.ebmwebsourcing.easycommons.properties.PropertiesException;

/**
 * <p>
 * A contextual file handler based on {@link PetalsFileHandler} and dumping message payload content.
 * </p>
 */
public class PetalsPayloadDumperFileHandler extends AbstractPetalsDumperFileHandler {

    private static final String DUMP_FILE_EXTENSION = ".xml";

    public PetalsPayloadDumperFileHandler() throws PropertiesException {
        super(DUMP_FILE_EXTENSION);
    }

    @Override
    protected void dumpMessageExchange(final MessageExchange exchange, final File toFile) throws Exception {

        assert exchange != null;
        assert toFile != null;

        final NormalizedMessage outNm = exchange.getMessage(JBI_OUT_MESSAGE);
        final NormalizedMessage inNm = exchange.getMessage(JBI_IN_MESSAGE);
        final NormalizedMessage faultNm = exchange.getFault();

        final Source source = faultNm != null ? faultNm.getContent()
                : outNm != null ? outNm.getContent()
                        : (inNm != null && exchange.getStatus().equals(ExchangeStatus.ACTIVE)) ? inNm.getContent()
                                : null;
    
        if (source != null) {
            try {
                this.sourceHelpers.toFile(source, toFile);
            } catch (final Exception e) {
                throw new Exception("Error while dumping exchange " + exchange.getExchangeId(), e);
            }
        }
    }
}
